import NodePath from 'path';
import * as zx from 'zx';
import { haveSamePath, insideDir } from './filesystem.js';
export const fetchSubmodules = async (options) => {
    const topLevel = (await zx.$ `git rev-parse --show-toplevel`).stdout.trim();
    let output = '';
    await insideDir(topLevel, async () => {
        output = (await zx.$ `git submodule status --recursive`).stdout;
    });
    let submodules = [];
    const submoduleRefs = output.split('\n').flatMap((rawLine) => {
        const line = rawLine.trim();
        if (line.length > 0) {
            let [commitHash, path] = line.split(' ');
            if (!!options.paths && !haveSamePath(options.paths, path)) {
                return [];
            }
            if (commitHash.startsWith('-')) {
                commitHash = commitHash.slice(1);
            }
            return {
                commitHash,
                path: NodePath.join(topLevel, path),
                gitModulePath: path,
            };
        }
        return [];
    });
    for (const { commitHash, path, gitModulePath } of submoduleRefs) {
        await insideDir(topLevel, async () => {
            let pathName = path.endsWith('/')
                ? path.substring(0, path.lastIndexOf('/'))
                : path;
            pathName = pathName.split('/').slice(-1)[0];
            const url = await zx.$ `git config --file .gitmodules --get submodule.${gitModulePath}.url`.then((output) => output.stdout.trim());
            submodules.push({ commitHash, path, url, gitModulePath });
        });
    }
    return submodules;
};
